import { others } from './svg';
import { cells } from './svg';
import { backgroundsArr, othersFiles, languageSvgStyles } from './western-world.utils.constants';
import {
  getPathByCell,
  getImageByUrl,
  getFlagsArray,
  buildImageForCanvasFromHtmlStr,
  getMaxSide,
  getMaskStyles,
} from './western-world.utils.helpers';

const parser = new DOMParser();

// forEach;
export const buildNewSvgImages = ({ allCells, language }) => {
  return Promise.all(
    allCells
      .filter((cell) => cell.id)
      .map((cell) => {
        return Promise.all([
          buildSvgAndImageByHtml({
            name: cell.animatedImage,
            elRequire: cells[cell.animatedImage],
            cell,
            language,
            withMask: true,
          }).then(({ img, svgTag }) => {
            window.cachedImages[language].svgTags[cell.animatedImage] = svgTag;
            window.cachedImages[language].svgTags[cell.id] = svgTag;
            window.cachedImages[language].svgImagesForCanvas[cell.animatedImage] = img;
            window.cachedImages[language].svgImagesForCanvas[cell.id] = img;
          }),
          buildSvgAndImageByHtml({
            name: cell.animatedImage,
            elRequire: cells[cell.animatedImage],
            cell,
            language,
            withMask: false,
            parser,
          }).then(({ img }) => {
            window.cachedImages[language].svgImagesWithoutMasks[cell.animatedImage] = img;
            window.cachedImages[language].svgImagesWithoutMasks[cell.id] = img;
          }),
        ]);
      })
  );
};

export const buildOtherSvgImages = ({ language }) => {
  return Promise.all(
    [...othersFiles, ...getFlagsArray('my'), ...getFlagsArray('enemy')].map((name) => {
      return buildSvgAndImageByHtml({
        name,
        elRequire: others[name],
        cell: null,
        language,
        withMask: false,
      }).then(({ img, svgTag }) => {
        window.cachedImages[language].svgTags[name] = svgTag;
        window.cachedImages[language].svgImagesForCanvas[name] = img;
        window.cachedImages[language].svgImagesWithoutMasks[name] = img;
      });
    })
  );
};

export const buildSvgAndImageByHtml = ({ name, elRequire, cell, language, withMask }) => {
  // get svg image from file and apply offsets;
  const doc = parser.parseFromString(elRequire.svgPath, 'image/svg+xml');
  const svgEl = doc.querySelector('svg');
  svgEl.classList.add(name);

  // create svg style element;
  const styleElement = svgEl.querySelector('svg > style');
  styleElement.insertAdjacentHTML('beforeend', `${languageSvgStyles} ${elRequire.scssPath}`);

  // create new <g> and move all content to it to apply mask;
  const gElement = document.createElementNS('http://www.w3.org/2000/svg', 'g');
  gElement.setAttribute('class', language);
  const allSvgImageContent = svgEl.querySelectorAll('svg > *:not(style)');
  [...allSvgImageContent].forEach((el) => gElement.appendChild(el));

  // create defs element;
  const defsElement = document.createElementNS('http://www.w3.org/2000/svg', 'defs');
  defsElement.innerHTML = elRequire.defs ? elRequire.defs.map((el) => el.value).join(' ') : '';

  if (withMask) {
    const maxMainSvgViewBox = svgEl.viewBox.baseVal.width;
    const maxCellSize = getMaxSide(cell);
    const cellPath = getPathByCell(name, cell, maxCellSize, maxMainSvgViewBox);
    const mask = `
            <mask id="mask-${name}">
              <use xlink:href="#path-border-${name}" class="use-mask"></use>
              <use xlink:href="#additional-mask" class="use-additional-mask"></use>
            </mask>
          `;
    gElement.setAttribute('mask', `url(#mask-${name})`);
    styleElement.insertAdjacentHTML('beforeend', getMaskStyles(cell));
    defsElement.insertAdjacentHTML('beforeend', `${cellPath} ${mask}`);

    // create use border path;
    const svgUseBorderElement = document.createElementNS('http://www.w3.org/2000/svg', 'use');
    svgUseBorderElement.setAttribute('xlink:href', `#path-border-${name}`);
    svgUseBorderElement.setAttribute('class', 'use-border');
    svgEl.appendChild(svgUseBorderElement);
  }

  svgEl.appendChild(defsElement);
  svgEl.appendChild(gElement);
  return buildImageForCanvasFromHtmlStr(svgEl.outerHTML).then((img) => {
    img.defs = elRequire.defs?.map(({ name, type }) => ({ name, type }));
    return { svgTag: svgEl.outerHTML, img };
  });
};

export const drawBackgrounds = ({ ctx, BACKGROUND_X, BACKGROUND_Y, BACKGROUND_WIDTH, BACKGROUND_HEIGHT, language }) => {
  backgroundsArr.forEach((fileName) =>
    ctx.drawImage(
      window.cachedImages[language].svgImagesForCanvas[fileName],
      BACKGROUND_X,
      BACKGROUND_Y,
      BACKGROUND_WIDTH,
      BACKGROUND_HEIGHT
    )
  );
};

export const drawBackgroundTopElements = ({
  ctx,
  BACKGROUND_X,
  BACKGROUND_Y,
  BACKGROUND_WIDTH,
  BACKGROUND_HEIGHT,
  language,
}) => {
  ctx.drawImage(
    window.cachedImages[language].svgImagesForCanvas['background-top-elements'],
    BACKGROUND_X,
    BACKGROUND_Y,
    BACKGROUND_WIDTH,
    BACKGROUND_HEIGHT
  );
};

export const drawNewAdditionalStuff = ({ ctx, SIZE_X_QUALITY, PADDING_LEFT, PADDING_TOP, language }) => {
  ctx.drawImage(
    window.cachedImages[language].svgImagesForCanvas['colony-arrows'],
    295 * SIZE_X_QUALITY + PADDING_LEFT,
    145 * SIZE_X_QUALITY + PADDING_TOP,
    1025 * SIZE_X_QUALITY,
    1025 * SIZE_X_QUALITY
  );
};

export const drawFlags = ({ allCells, ctx, game, myIdKey, SIZE_X_QUALITY, PADDING_LEFT, PADDING_TOP, language }) => {
  if (!game.players) {
    return;
  }
  const myPlayer = game.players[myIdKey];
  const allMyCompanies = myPlayer?.firms;
  const allEnemyCompanies = game.players.order.reduce((previousValue, playerIdKey) => {
    if (playerIdKey === myPlayer?.user?.id_key) {
      return previousValue;
    }
    game.players[playerIdKey].firms.forEach((firmId) => {
      previousValue[firmId] = playerIdKey;
    });
    return previousValue;
  }, {});
  allCells
    .filter((cell) => cell.id)
    .forEach((cell) => {
      const flagX = cell.flag ? cell.flag.x : cell.location.x1 + (cell.location.x2 - cell.location.x1) / 2;
      const flagY = cell.flag ? cell.flag.y : cell.location.y1;
      let flag;
      let avatar;
      if (allEnemyCompanies[cell.id]) {
        flag = window.cachedImages[language].svgImagesForCanvas[`enemy-flag-stars-${game.stocks[cell.id].stars}`];
        avatar = window.cachedImages[language].avatarsMapForCanvas[allEnemyCompanies[cell.id]];
      }
      if (allMyCompanies?.includes(cell.id)) {
        flag = window.cachedImages[language].svgImagesForCanvas[`my-flag-stars-${game.stocks[cell.id].stars}`];
        avatar = window.cachedImages[language].avatarsMapForCanvas[myPlayer.user.id_key];
      }
      if (!flag || !avatar) {
        return;
      }
      ctx.drawImage(
        flag,
        (cell.location.x1 + flagX - 53) * SIZE_X_QUALITY + PADDING_LEFT,
        (cell.location.y1 + flagY - 60) * SIZE_X_QUALITY + PADDING_TOP,
        64 * SIZE_X_QUALITY,
        64 * SIZE_X_QUALITY
      );
      ctx.drawImage(
        avatar,
        (cell.location.x1 + flagX - 22.5) * SIZE_X_QUALITY + PADDING_LEFT,
        (cell.location.y1 + flagY - 35) * SIZE_X_QUALITY + PADDING_TOP,
        15 * SIZE_X_QUALITY,
        15 * SIZE_X_QUALITY
      );
    });
};

export const drawAllNewCells = ({ allCells, ctx, SIZE_X_QUALITY, PADDING_LEFT, PADDING_TOP, language }) => {
  allCells
    .filter((cell) => cell.id)
    .forEach((cell) => {
      const max = getMaxSide(cell);
      ctx.drawImage(
        window.cachedImages[language].svgImagesForCanvas[cell.id],
        cell.location.x1 * SIZE_X_QUALITY + PADDING_LEFT,
        cell.location.y1 * SIZE_X_QUALITY + PADDING_TOP,
        max * SIZE_X_QUALITY,
        max * SIZE_X_QUALITY
      );
    });
};

let currentPromise;
let isResolved = false;
export const buildPlayerAvatars = ({ avatars, language }) => {
  if (!avatars) {
    return Promise.resolve();
  }
  if (currentPromise && !isResolved) {
    return currentPromise;
  }
  // if currentPromise and all avatars are same;
  if (
    currentPromise &&
    isResolved &&
    avatars.every(({ id_key }) => window.cachedImages[language].avatarsMapForCanvas[id_key])
  ) {
    return currentPromise;
  }
  isResolved = false;
  currentPromise = Promise.all(avatars.map(({ iconBase64 }) => getImageByUrl(iconBase64))).then((data) => {
    isResolved = true;
    data.forEach((image, index) => {
      window.cachedImages[language].avatarsMapForCanvas[avatars[index].id_key] = image;
    });
  });
  return currentPromise;
};
